# -*- coding: utf-8 -*-
"""
    wakatime
    ~~~~~~~~

    Common interface to the WakaTime api.
    http://wakatime.com

    :copyright: (c) 2013 Alan Hamlett.
    :license: BSD, see LICENSE for more details.
"""


__all__ = ['main']


from .main import execute
